/**
 * 
 */
package sim.geometry.space;

import java.util.List;

import sim.exception.SRuntimeException;
import sim.geometry.SGeometry;
import sim.geometry.SRay;
import sim.math.SVector3d;

/**
 * L'interface <b>SGeometrySpace</b> reprsentant un espace de rengement des gomtries. 
 * Cet espace permet d'organiser les gomtries afin d'effectuer le calculs de l'intersection avec les gomtries plus efficacement.
 * 
 * @author Simon Vzina
 * @since 2015-01-10
 * @version 2015-12-29
 */
public interface SGeometrySpace {

	/**
	 * Mthode qui ajoute une gomtrie  l'espace.
	 * @param geometry - La gomtrie  ajouter  l'espace.
	 */
	public void addGeometry(SGeometry geometry);
	
	/**
	 * Mthode qui ajoute une liste de gomtries  l'espace.
	 * @param list - La liste de gomtries  ajouter  l'espace.
	 */
	public void addGeometry(List<SGeometry> list);
	
	/**
	 * Mthode qui value l'intersection la plus prs entre un rayon et les diffrentes gomtries de l'espace ne dpassant pas un certain temps maximal.
	 * @param ray - Le rayon  intersecter avec les gomtries de l'espace.
	 * @param t_max - Le temps maximal.
	 * @return Un nouveau rayon avec les proprits de l'intersection la plus prs ne dpassant pas le temps maximal. Si <b>aucune intersection</b> respectant ces contraintes n'a peut tre ralise, le <b>rayon d'origine sera retourn</b> tant un rayon sans intersection.
	 * @throws SRuntimeException Si le rayon a dj intersect une autre gomtrie.
	 * @throws SRuntimeException Si la valeur de t_max est ngative.
	 * @throws SRuntimeException Si l'espace des gomtries n'a pas t pralablement initialis.
	 */
	public SRay nearestIntersection(SRay ray, double t_max) throws SRuntimeException;
		
	/**
	 * Mthode qui value l'intersection la plus prs entre un rayon et les diffrentes gomtries <b>opaque</b> de l'espace ne dpassant pas un certain temps maximal.
	 * Une liste des gomtries transparentes sera ordonne en <b>ordre inverse d'apparition</b> (ordre dcroissant) et la <b>liste dbutera par une gomtrie opaque</b> s'il y a eu intersection avec une gomtrie opaque.
	 * @param ray - Le rayon  intersecter avec les gomtries de l'espace.
	 * @param t_max - Le temps maximal.
	 * @return Une liste des gomtries transparentes intersectes en <b>ordre dcroissant</b> dbutant par une gomtrie opaque s'il y a eu lieu. La liste sera <b>vide</b> s'il n'y a <b>aucune intersection</b> sur le temps maximal.
	 * @throws SRuntimeException Si le rayon a dj intersect une autre gomtrie.
	 * @throws SRuntimeException Si la valeur de t_max est ngative.
	 * @throws SRuntimeException Si l'espace des gomtries n'a pas t pralablement initialis.
	 */
	public List<SRay> nearestOpaqueIntersection(SRay ray, double t_max) throws SRuntimeException;
	
	/**
	 * Mthode pour obtenir la liste des gomtries o la position du vecteur <i>v</i> se retrouve  l'intrieur.
	 * Si la liste est vide, c'est que le vecteur <i>v</i> se retrouve  l'intrieur d'aucune gomtrie.
	 * @param v - La position d'un point de l'espace.
	 * @return La liste des gomtries o le vecteur <i>v</i> se retrouve  l'intrieur.
	 * @throws SRuntimeException Si l'espace des gomtries n'a pas t pralablement initialis.
	 */
	public List<SGeometry> listInsideGeometry(SVector3d v) throws SRuntimeException;
	
	/**
	 * Mthode pour initialiser l'espace des gomtries. L'espace ne sera pas oprationnel 
	 * pour effectuer des tests d'intersection si cette mthode n'est pas active avant.
	 */
	public void initialize();
	
}//fin interface SGeometrySpace
