/**
 * 
 */
package sim.graphics.light;

import sim.exception.SRuntimeException;
import sim.math.SVector3d;

/**
 * L'interface <b>SAttenuatedLight</b> reprsentant une source de lumire qui peut s'attnuer en fonction de la distance entre la source de lumire et la position claire.
 * 
 * @author Simon Vzina
 * @since 2015-02-06
 * @version 2016-02-28
 */
public interface SAttenuatedLight extends SLight {

	/**
	 * Mthode pour obtenir la position de la source de lumire.
	 * 
	 * @return La position de la source de lumire.
	 */
	public SVector3d getPosition();
	
	/**
	 * Mthode pour obtenir l'orientation de la source de lumire en fonction de la position du point  illuminer.
	 * 
	 * @param position_to_illuminate - La position  illuminer
	 * @return L'orientation de la source de lumire (unitaire).
	 * @throws SRuntimeException Si la position  illuminer est situe sur la source de lumire.
	 */
	public SVector3d getOrientation(SVector3d position_to_illuminate) throws SRuntimeException;
	
	/**
	 * <p>
	 * Mthode pour obtenir le facteur d'attnuation A qui dpend de la distance d 
	 * entre le point  clairer et la source de lumire. La formule utilise est 
	 * <ul> A = 1 / ( Ccst + Clin*d + Cquad*d*d ).</ul>
	 * Si d > 1, alors A < 1.
	 * </p>
	 * <p>
	 * Cependant, si la <b>distance d est infrieure  1</b>, le facteur d'attnuation A peut tre
	 * suprieur  1.0 (A > 1) ce qui donnera une intensit accrue  la couleur de la source de lumire.
	 * Le choix de normalisation de la couleur devra tre judicieusement afin de crer l'effet visuel dsir.
	 * </p>
	 * 
	 * @param position_to_illuminate - La position  clairer.
	 * @return Le facteur d'attnuation A  la position  clairer.  
	 * @throws SRuntimeException Si le facteur d'attnuation A tend vers l'infini (dnominateur tant gal  zro).
	 */
	public double attenuation(SVector3d position_to_illuminate) throws SRuntimeException;
	
	/**
	 * <p>
	 * Mthode pour obtenir le facteur d'amplification de la source de lumire.
	 * Cependant, une couleur amplifie peut faire excder un canal de couleur au-del de 1.0.
	 * Dans cette situation, il sera important de bien choisir l'algorithme de normalisation des couleurs
	 * afin d'obtenir le rsultat dsir .
	 * </p>
	 * 
	 * <p>
	 * En utilisant cette option dans un <i>shader</i>, cela permet d'attribuer plus d'importance  une source de lumire.
	 * </p>
	 * 
	 * @return Le facteur d'amplification de la source de lumire.
	 * @see SColor
	 */
	public double amplification();
	
}//fin interface SAttenuatedLight
