/**
 * 
 */
package sim.parser.model.agp;

import java.util.ArrayList;

/**
 * Classe qui reprsente une squence de point.
 * 
 * @author Simon Vzina
 * @since 2015-08-01
 * @version 2015-08-01
 */
public class SPointSequence {

  private final ArrayList<SPoint> sequence; //la squence de point
  
  /**
   * Constructeur d'une squence de point avec un nombre d'lments  l'intrieur qui devront tre dfinis. 
   * @param nb - Le nombre de points  l'intrieur de la squence.
   */
  public SPointSequence(int nb)
  {
    sequence = new ArrayList<SPoint>();
    
    //Ajouter des points non dfini  l'intrieur de la squence
    for(int i=0; i<nb; i++)
      sequence.add(new SPoint());
  }

  /**
   * Mthode pour obtenir un point  un index spcifique de la squence de point. L'index 0 correspond
   * au premier point.
   * @param index - L'index du point.
   * @return Le point  l'index spcifique
   * @throws IndexOutOfBoundsException Si l'index est  l'extrieur des bornes suivantes (index < 0 || index >= size()) 
   */
  public SPoint getPoint(int index)throws IndexOutOfBoundsException 
  {
    return sequence.get(index);
  }
 
  /**
   * Mthode pour obtenir la taille de la squence.
   * @return La taille de la squence.
   */
  public int size()
  {
    return sequence.size();
  }
  
  /**
   * Mthode pour dfinir la coordonne x d'un point  un index particulier.
   * @param index - L'index du point dont la coordonne x sera modifie.
   * @throws IndexOutOfBoundsException Si l'index est  l'extrieur des bornes suivantes (index < 0 || index >= size()) 
   */
  public void setX(int index, double x)throws IndexOutOfBoundsException 
  {
    sequence.get(index).setX(x);
  }
  
  /**
   * Mthode pour dfinir la coordonne y d'un point  un index particulier.
   * @param index - L'index du point dont la coordonne y sera modifie.
   * @throws IndexOutOfBoundsException Si l'index est  l'extrieur des bornes suivantes (index < 0 || index >= size()) 
   */
  public void setY(int index, double y)throws IndexOutOfBoundsException 
  {
    sequence.get(index).setY(y);
  }
  
  /**
   * Mthode pour dfinir la coordonne z d'un point  un index particulier.
   * @param index - L'index du point dont la coordonne z sera modifie.
   * @throws IndexOutOfBoundsException Si l'index est  l'extrieur des bornes suivantes (index < 0 || index >= size()) 
   */
  public void setZ(int index, double z)throws IndexOutOfBoundsException 
  {
    sequence.get(index).setZ(z);
  }
  
}//fin de la classe SPointSequence
