/**
 * 
 */
package sim.geometry;

import org.junit.Assert;
import org.junit.Test;

import sim.math.SVector3d;


/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>STransformableGeometry</b>.
 * 
 * @author Simon Vzina
 * @since 2016-04-19
 * @version 2020-08-12
 */
public class STransformableGeometryTest {

  /**
   * Test de la mthode isInside(SVector3d v) avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il n'y a pas de transformation.
   * Plusieurs tests seront raliss o le point test sera  l'intrieur et  l'extrieur de la gomtrie transforme.
   */
  @Test
  public void isInsideTest1()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
    
  	SVector3d scale = new SVector3d(1.0, 1.0, 1.0);
  	SVector3d rotation = new SVector3d(0.0, 0.0, 0.0);
  	SVector3d translation = new SVector3d(0.0, 0.0, 0.0);
	  
    // Avec la transformation, la diagonale du cube dans le plan xy sera gal  (2^2 + 2^2)^1/2 = 2.828427...
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
    
    // Test #1 : Le centre du cube translat
    Assert.assertEquals(true, transformable_geometry.isInside(new SVector3d(1.0, 0.0, 0.0)));
    
    // Test #2 : Une extrmint du cube sous une rotation ( l'intrieur)
    Assert.assertEquals(true, transformable_geometry.isInside(new SVector3d(-1.0, -1.0, -1.0)));
    
    // Test #3 : Une extrmint du cube sous une rotation ( l'extrieur)
    Assert.assertEquals(false, transformable_geometry.isInside(new SVector3d(3.0, 0.0, 0.0)));
    
    // Test #4 : Un vecteur non transform
    Assert.assertEquals(false, transformable_geometry.isInside(new SVector3d(3.0, 3.0, 3.0)));	
  }
  
  /**
   * Test de la mthode isInside(SVector3d v) avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il y a plusieurs transformations.
   * Plusieurs tests seront raliss o le point test sera  l'intrieur et  l'extrieur de la gomtrie transforme.
   */
  @Test
  public void isInsideTest2()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 2.0);
    
  	SVector3d scale = new SVector3d(2.0, 2.0, 2.0);
  	SVector3d rotation = new SVector3d(0.0, 0.0, 45.0);
  	SVector3d translation = new SVector3d(10.0, 10.0, 10.0);
	  
    // Avec la transformation, la diagonale du cube dans le plan xy sera gal  (2^2 + 2^2)^1/2 = 2.828427...
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
    
    // Test #1 : Le centre du cube translat
    Assert.assertEquals(true, transformable_geometry.isInside(new SVector3d(10.0, 10.0, 10.0)));
    
    // Test #2 : Une extrmint du cube sous une rotation ( l'intrieur)
    Assert.assertEquals(true, transformable_geometry.isInside(new SVector3d(10.0, 10.0 + 2.82842, 10.0)));
    
    // Test #3 : Une extrmint du cube sous une rotation ( l'extrieur)
    Assert.assertEquals(false, transformable_geometry.isInside(new SVector3d(10.0, 10.0 + 2.82843, 10.0)));
    
    // Test #4 : Un vecteur non transform
    Assert.assertEquals(false, transformable_geometry.isInside(new SVector3d(0.0, 0.0, 0.0)));	
  }

  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o aucune transformation n'est requise. 
   * Dans ce test, il y <u>aura intersection</u>.
   */
  @Test
  public void intersectionTest1a()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
  	  
  	SVector3d scale = new SVector3d(1.0, 1.0, 1.0);
  	SVector3d rotation = SVector3d.ZERO;
  	SVector3d translation = SVector3d.ZERO;
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(-5.0, 1.0, 1.0);
  	SVector3d ray_direction = new SVector3d(1.0, 0.0, 0.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	SRay expected = ray.intersection(transformable_geometry, new SVector3d(-1.0, 0.0, 0.0), 3.0);
  	
  	Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o aucune transformation n'est requise. 
   * Dans ce test, il <u>n'y aura pas d'intersection</u>.
   */
  @Test
  public void intersectionTest1b()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 3.0);
  	  
  	SVector3d scale = new SVector3d(1.0, 1.0, 1.0);
  	SVector3d rotation = SVector3d.ZERO;
  	SVector3d translation = SVector3d.ZERO;
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(-5.0, 1.0, 1.0);
  	SVector3d ray_direction = new SVector3d(1.0, -3.0, 0.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	// Il n'y aura pas d'intersection.
  	SRay expected = ray;
  	
  	Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il y aura uniquement une transformation de translation. 
   * Dans ce test, il y <u>aura intersection</u>.
   */
  @Test
  public void intersectionTest2a()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
  	  
  	SVector3d scale = new SVector3d(1.0, 1.0, 1.0);
  	SVector3d rotation = SVector3d.ZERO;
  	SVector3d translation = new SVector3d(0.0, 0.0, -5.0);
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(-5.0, 1.0, 1.0);
  	SVector3d ray_direction = new SVector3d(1.0, 0.0, -2.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	SRay expected = ray.intersection(transformable_geometry, new SVector3d(-1.0, 0.0, 0.0), 3.0);
  	
  	Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il y aura uniquement une transformation de translation. 
   * Dans ce test, il y <u>n'y aura pas intersection</u>.
   */
  @Test
  public void intersectionTest2b()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
      
    SVector3d scale = new SVector3d(1.0, 1.0, 1.0);
    SVector3d rotation = SVector3d.ZERO;
    SVector3d translation = new SVector3d(0.0, 0.0, -5.0);
      
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
      
    SVector3d ray_position = new SVector3d(-5.0, 1.0, 1.0);
    SVector3d ray_direction = new SVector3d(1.0, 3.0, -2.0);
      
    SRay ray = new SRay(ray_position, ray_direction, 1.0);
    
    SRay calculated = transformable_geometry.intersection(ray);
    
    // Il n'y aura pas d'intersection.
    SRay expected = ray;
    
    Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il y aura uniquement une transformation d'homothtie (back coverage). 
   * Dans ce test, il y <u>aura intersection</u>.
   */
  @Test
  public void intersectionTest3a()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
      
    SVector3d scale = new SVector3d(3.0, 4.0, 5.0);
    SVector3d rotation = SVector3d.ZERO;
    SVector3d translation = SVector3d.ZERO;
      
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
      
    SVector3d ray_position = new SVector3d(-5.0, 1.0, 1.0);
    SVector3d ray_direction = new SVector3d(1.0, 0.0, -2.0);
      
    SRay ray = new SRay(ray_position, ray_direction, 1.0);
    
    SRay calculated = transformable_geometry.intersection(ray);
    
    SRay expected = ray.intersection(transformable_geometry, new SVector3d(0.0, 0.0, -1.0), 5.5);
    
    Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans le cas o il y aura uniquement une transformation d'homothtie (back coverage). 
   * Dans ce test, il y <u>n'y aura pas intersection</u>.
   */
  @Test
  public void intersectionTest3b()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
    
    SVector3d scale = new SVector3d(3.0, 4.0, 5.0);
    SVector3d rotation = SVector3d.ZERO;
    SVector3d translation = SVector3d.ZERO;
      
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
      
    SVector3d ray_position = new SVector3d(-11.9, 1.0, 1.0);
    SVector3d ray_direction = new SVector3d(1.0, 0.0, -2.0);
      
    SRay ray = new SRay(ray_position, ray_direction, 1.0);
    
    SRay calculated = transformable_geometry.intersection(ray);
    
    // Il n'y aura pas d'intersection.
    SRay expected = ray;
    
    Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans un cas gnral (back coverage). 
   * Dans ce test, il y <u>aura intersection</u>.
   */
  @Test
  public void intersectionTest4a()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 3.0);
      
    SVector3d scale = new SVector3d(5.0, 5.0, 5.0);
    SVector3d rotation = new SVector3d(0.0, 0.0, 45.0);
    SVector3d translation = new SVector3d(0.0, 0.0, -2.0);
      
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
      
    SVector3d ray_position = new SVector3d(3.0, 3.0, 8.0);
    SVector3d ray_direction = new SVector3d(1.0, 0.0, -1.0);
      
    SRay ray = new SRay(ray_position, ray_direction, 1.0);
    
    SRay calculated = transformable_geometry.intersection(ray);
    
    SRay expected = ray.intersection(transformable_geometry, new SVector3d(0.0, 0.0, 1.0), 2.5);
    
    Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans un cas gnral. 
   * Dans ce test, il <u>n'y aura pas d'intersection</u>.
   */
  @Test
  public void intersectionTest4b()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 3.0);
  	  
  	SVector3d scale = new SVector3d(5.0, 5.0, 5.0);
  	SVector3d rotation = new SVector3d(0.0, 0.0, 45.0);
  	SVector3d translation = new SVector3d(0.0, 0.0, -10.0);
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(5.0, 5.0, 9.0);
  	SVector3d ray_direction = new SVector3d(5.0, 0.0, -1.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	// Il n'y a pas d'intersection.
  	SRay expected = ray;
  	
  	Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans un cas gnral. 
   * Dans ce test, il y aura intersection car la rotation du cube selon l'axe z est faible. 
   * La normale "normalise" sera la mme dans l'espace monde et objet.
   * 
   * Ce test ne vrifie pas la transformation de la normale.
   * 
   */
  @Test
  public void intersectionTest5a()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
  	  
  	SVector3d scale = new SVector3d(5.0, 5.0, 5.0);
  	SVector3d rotation = new SVector3d(0.0, 0.0, 5.0);
  	SVector3d translation = new SVector3d(0.0, 0.0, -5.0);
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(7.0, 7.0, 7.0);
  	SVector3d ray_direction = new SVector3d(1.0, 0.0, -1.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	SRay expected = ray.intersection(transformable_geometry, new SVector3d(0.0, 0.0, 1.0), 2.0);
  	
  	Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans un cas gnral. 
   * Dans ce test, il <u>n'y aura intersection</u> car la rotation du cube selon l'axe z est prononce.
   */
  @Test
  public void intersectionTest5b()
  {
    SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
      
    SVector3d scale = new SVector3d(5.0, 5.0, 5.0);
    SVector3d rotation = new SVector3d(0.0, 0.0, 65.0);
    SVector3d translation = new SVector3d(0.0, 0.0, -5.0);
      
    STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
      
    SVector3d ray_position = new SVector3d(7.0, 7.0, 7.0);
    SVector3d ray_direction = new SVector3d(1.0, 0.0, -1.0);
      
    SRay ray = new SRay(ray_position, ray_direction, 1.0);
    
    SRay calculated = transformable_geometry.intersection(ray);
    
    // Il n'y a pas d'intersection.
    SRay expected = ray;
    
    Assert.assertEquals(expected, calculated);
  }
  
  /**
   * Test de la mthode intersection avec l'usage de la gomtrie <b>SCubeGeometry</b> dans un cas gnral. 
   * Dans ce test, il y <u>aura intersection</u> et la normale de l'espace objet devra tre transforme vers l'espace monde et normalise.
   */
  @Test
  public void intersectionTest6()
  {
  	SCubeGeometry geometry = new SCubeGeometry(new SVector3d(0.0, 0.0, 0.0), 4.0);
  	  
  	SVector3d scale = new SVector3d(5.0, 5.0, 5.0);
  	SVector3d rotation = new SVector3d(0.0, 45.0, 45.0);
  	SVector3d translation = new SVector3d(0.0, 0.0, -10.0);
  	  
  	STransformableGeometry transformable_geometry = new STransformableGeometry(geometry, scale, rotation, translation);
  	  
  	SVector3d ray_position = new SVector3d(5.0, 5.0, 2.0);
  	SVector3d ray_direction = new SVector3d(1.0, 0.0, -3.0);
  	  
  	SRay ray = new SRay(ray_position, ray_direction, 1.0);
  	
  	SRay calculated = transformable_geometry.intersection(ray);
  	
  	SRay expected = ray.intersection(transformable_geometry, new SVector3d(0.5, 0.5, 0.7071067811865476), 2.149656228075546);
  	
  	Assert.assertEquals(expected, calculated);
  }
    
}//fin de la classe STransformableGeometry
